@@ -179,8 +179,8 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                          pars=pars, ngroups=ngroups, J=J, itemloc=itemloc,
                          Theta=Theta, PrepList=PrepList, dentype=dentype, lrPars=lrPars,
                          specific=specific, sitems=sitems, CUSTOM.IND=CUSTOM.IND,
-                         constrain=constrain, EHPrior=NULL, Data=Data, method=Moptim,
-                         control=control, hessian=list$SE,
+                         constrain=constrain, EHPrior=NULL, Data=Data, omp_threads=list$omp_threads,
+                         method=Moptim, control=control, hessian=list$SE,
                          lower=lower, upper=upper), silent=TRUE)
         cycles <- as.integer(opt$counts[1L])
         longpars[est] <- opt$par
@@ -198,11 +198,12 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                 rlist[[g]] <- Estep.bfactor(pars=pars[[g]], tabdata=Data$tabdatalong, freq=Data$Freq[[g]],
                                             Theta=Theta, prior=prior[[g]],
                                             Priorbetween=Priorbetween[[g]], specific=specific,
-                                            sitems=sitems, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)
+                                            sitems=sitems, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
+                                            omp_threads=list$omp_threads)
             } else {
                 rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=Data$tabdatalong, freq=Data$Freq[[g]],
                                          CUSTOM.IND=CUSTOM.IND, Theta=Theta,
-                                         prior=Prior[[g]], itemloc=itemloc)
+                                         prior=Prior[[g]], itemloc=itemloc, omp_threads=list$omp_threads)
             }
             LL <- LL + sum(Data$Freq[[g]]*log(rlist[[g]]$expected))
         }
@@ -239,7 +240,8 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
             Elist <- Estep(pars=pars, Data=Data, gTheta=gTheta, prior=prior, Prior=Prior,
                            Priorbetween=Priorbetween, specific=specific, sitems=sitems,
                            ngroups=ngroups, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
-                           dentype=dentype, rlist=rlist, full=full, Etable=list$Etable)
+                           dentype=dentype, rlist=rlist, full=full, Etable=list$Etable,
+                           omp_threads=list$omp_threads)
             rlist <- Elist$rlist; LL <- Elist$LL
             if(any(ANY.PRIOR)){
                 LP <- 0
@@ -273,8 +275,17 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                 if(dentype == 'bfactor'){
                     pars[[g]][[J+1L]]@rrb <- rlist[[g]]$r2
                     pars[[g]][[J+1L]]@rrs <- rlist[[g]]$r3
-                } else pars[[g]][[J+1L]]@rr <- rowSums(rlist[[g]]$r1) / J
+                } else {
+                    pars[[g]][[J+1L]]@rr <- rowSums(rlist[[g]]$r1) / J
+                    if(dentype %in% c('EHW', 'Davidian') ||
+                       (dentype == 'custom' && pars[[g]][[J+1L]]@standardize))
+                        pars[[g]][[J+1L]]@rr <- standardizeQuadrature(gTheta[[g]],
+                                                        nq=pars[[g]][[J+1L]]@rr,
+                                                        estmean=pars[[g]][[J+1L]]@est['MEAN_1'],
+                                                        estsd=pars[[g]][[J+1L]]@est['COV_11'])
+                }
             }
+            Estep.time <- Estep.time + proc.time()[3L] - start
             start <- proc.time()[3L]
             preMstep.longpars2 <- preMstep.longpars
             preMstep.longpars <- longpars
@@ -339,7 +350,7 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                             Elist <- Estep(pars=pars, Data=Data, gTheta=gTheta, prior=prior, Prior=Prior,
                                            Priorbetween=Priorbetween, specific=specific, sitems=sitems,
                                            ngroups=ngroups, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
-                                           dentype=dentype, rlist=rlist, full=full)
+                                           dentype=dentype, rlist=rlist, full=full, omp_threads=list$omp_threads)
                             if(Elist$LL <= collectLL[cycles]){
                                 accel <- (accel - 1) / 2
                                 count <- count + 1L
@@ -387,10 +398,10 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
             Elist <- Estep(pars=pars, Data=Data, gTheta=gTheta, prior=prior, Prior=Prior,
                            Priorbetween=Priorbetween, specific=specific, sitems=sitems,
                            ngroups=ngroups, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
-                           dentype=dentype, rlist=rlist, full=full, Etable=list$Etable)
+                           dentype=dentype, rlist=rlist, full=full, Etable=list$Etable,
+                           omp_threads=list$omp_threads)
             rlist <- Elist$rlist; LL <- Elist$LL
         }
-        if(verbose && !is.nan(TOL) && !is.na(TOL)) cat("\n")
         if(cycles == NCYCLES){
             if(list$message)
                 message('EM cycles terminated after ', cycles, ' iterations.')
@@ -441,7 +452,10 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                            Theta=gTheta[[group]], EM = TRUE,
                            pars=pars[[group]], tabdata=Data$tabdatalong,
                            freq=Data$Freq[[group]], prior=Prior[[group]],
-                           itemloc=itemloc, estHess=TRUE)
+                           itemloc=itemloc,
+                           bfactor_info=if(dentype == 'bfactor')
+                               list(specific=specific, sitems=sitems, nfact=nfact) else NULL,
+                           estHess=TRUE)
             ind2 <- ind1 + length(deriv$grad) - 1L
             h[ind1:ind2, ind1:ind2] <- pars[[group]][[i]]@hessian <- deriv$hess
             ind1 <- ind2 + 1L
@@ -493,7 +507,7 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                                        specific=specific, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
                                        prior=prior, Priorbetween=Priorbetween, Prior=Prior,
                                        PrepList=PrepList, ANY.PRIOR=ANY.PRIOR, DERIV=DERIV,
-                                       SLOW.IND=list$SLOW.IND, Norder=list$Norder)
+                                       SLOW.IND=list$SLOW.IND, Norder=list$Norder, omp_threads=list$omp_threads)
             } else {
                 zero_g <- SE.Oakes(0L, pars=pars, L=L, constrain=constrain, delta=0,
                                    est=est, shortpars=shortpars, longpars=longpars,
@@ -503,7 +517,7 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                                    specific=specific, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
                                    prior=prior, Priorbetween=Priorbetween, Prior=Prior,
                                    PrepList=PrepList, ANY.PRIOR=ANY.PRIOR, DERIV=DERIV,
-                                   SLOW.IND=list$SLOW.IND, Norder=1L)
+                                   SLOW.IND=list$SLOW.IND, Norder=1L, omp_threads=list$omp_threads)
                 missing_info <- mySapply(seq_len(length(shortpars)), SE.Oakes,
                                        pars=pars, L=L, constrain=constrain, delta=list$delta,
                                        est=est, shortpars=shortpars, longpars=longpars,
@@ -513,7 +527,7 @@ EM.group <- function(pars, constrain, Ls, Data, PrepList, list, Theta, DERIV, so
                                        specific=specific, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND,
                                        prior=prior, Priorbetween=Priorbetween, Prior=Prior,
                                        PrepList=PrepList, ANY.PRIOR=ANY.PRIOR, DERIV=DERIV,
-                                       SLOW.IND=list$SLOW.IND, zero_g=zero_g, Norder=1L)
+                                       SLOW.IND=list$SLOW.IND, zero_g=zero_g, Norder=1L, omp_threads=list$omp_threads)
             }
             if(list$symmetric) missing_info <- (missing_info + t(missing_info))/2
             pars <- reloadPars(longpars=longpars, pars=pars,
